#!/usr/bin/perl -w

  # use perl;
  use strict;

  #
  # Local Prototypes.
  #
  sub read_file($);
  sub write_file($@);

  #
  # Set up some variables we'll use later.
  #
  my $file		= '/etc/httpd/conf/httpd.conf';
  my @file		= read_file($file);
  my $state		= {
	'auth'		=> undef,
	'line'		=> undef,
	'ssl_s'		=> undef,
	'ssl_e'		=> undef,
	'aliasgoner'	=> undef,
  };
  my $translation_table	= {
	'access_module'		=> {
		'name'		=> 'authz_host_module',
		'module'	=> 'mod_authz_host.so',
	},
	'auth_anon_module'	=> {
		'name'		=> 'authn_anon_module',
		'module'	=> 'mod_authn_anon.so',
	},
	'auth_dbm_module'	=> {
		'name'		=> 'authn_dbm_module',
		'module'	=> 'mod_authn_dbm.so',
	},
	'auth_module'		=> {
		'name'		=> 'auth_basic_module',
		'module'	=> 'mod_auth_basic.so',
	},
	'imap_module'		=> {
		'name'		=> 'imagemap_module',
		'module'	=> 'mod_imagemap.so',
	},
  };

  #
  # Loop over the file...
  #
  for (my $i = 0; $i < @file; $i++) {

    #
    # ... update any 1:1 module name changes.
    #
    if ($file[$i] =~ m/^LoadModule\s+([^\s]+)\s+\/usr\/libexec\/apache\/([^\s]+)$/) {
      my $a	= $1;
      my $b	= $2;

      # Update $state if we found mod_auth.so.
      if ($a eq 'auth_module') { $state->{'auth'} = 1; $state->{'line'}	= $i; }

      # Update this line.
      if ($translation_table->{$a}) {
        $file[$i]	= sprintf('LoadModule %s /usr/libexec/apache/%s',
				$translation_table->{$a}->{'name'},
				$translation_table->{$a}->{'module'});
      }
    }

    #
    # ... we're going to replace this block with another below.
    #
    if (($file[$i] eq '<IfModule mod_ssl.c>') &&
        ($file[$i+1] =~ m/^\s*Include\s+\/etc\/httpd\/conf\/ssl.conf/) &&
        ($file[$i+2] eq '</IfModule>'))
    {
      $state->{'ssl_s'}	= $i;
      $state->{'ssl_e'}	= $i + 2;
    }

    #
    # ... this line is duplicated in extra/httpd-autoindex.conf.
    #
    if ($file[$i] eq 'Alias /icons/ "/usr/share/apache/icons/"') {
      $state->{'aliasgoner'}	= $i;
    }
  }

  #
  # Splice in our include block.
  #
  if ($state->{'ssl_s'} && $state->{'ssl_e'}) {
    my @new	= (
	'# Server-pool management (MPM specific)',
	'#Include /etc/httpd/conf/extra/httpd-mpm.conf',
	'',
	'# Multi-language error messages',
	'#Include /etc/httpd/conf/extra/httpd-multilang-errordoc.conf',
	'',
	'# Fancy directory listings',
	'Include /etc/httpd/conf/extra/httpd-autoindex.conf',
	'',
	'# Language settings',
	'Include /etc/httpd/conf/extra/httpd-languages.conf',
	'',
	'# User home directories',
	'Include /etc/httpd/conf/extra/httpd-userdir.conf',
	'',
	'# Real-time info on requests and configuration',
	'#Include /etc/httpd/conf/extra/httpd-info.conf',
	'',
	'# Virtual hosts',
	'#Include /etc/httpd/conf/extra/httpd-vhosts.conf',
	'',
	'# Local access to the Apache HTTP Server Manual',
	'#Include /etc/httpd/conf/extra/httpd-manual.conf',
	'',
	'# Distributed authoring and versioning (WebDAV)',
	'#Include /etc/httpd/conf/extra/httpd-dav.conf',
	'',
	'# Various default settings',
	'Include /etc/httpd/conf/extra/httpd-default.conf',
	'',
	'# Secure (SSL/TLS) connections',
	'Include /etc/httpd/conf/extra/httpd-ssl.conf',
	'',
	'#',
	'# Note: The following must must be present to support',
	'#       starting without SSL on platforms with no /dev/random equivalent',
	'#       but a statically compiled-in mod_ssl.',
	'#',
	'<IfModule ssl_module>',
	'SSLRandomSeed startup builtin',
	'SSLRandomSeed connect builtin',
	'</IfModule>',
    );

    splice @file, $state->{'ssl_s'}, 3, @new;
  }

  #
  # Remove this line.
  #
  if ($state->{'aliasgoner'}) { splice @file, $state->{'aliasgoner'}, 1; }

  #
  # Finally, finish our module-FOO.
  #
  if ($state->{'auth'} && $state->{'line'}) {
    my @new	= ();

    push @new, 'LoadModule authn_file_module /usr/libexec/apache/mod_authn_file.so'
	if (-f '/usr/libexec/apache/mod_authn_file.so');
    push @new, 'LoadModule authz_user_module /usr/libexec/apache/mod_authz_user.so'
	if (-f '/usr/libexec/apache/mod_authz_user.so');
    push @new, 'LoadModule authz_groupfile_module /usr/libexec/apache/mod_authz_groupfile.so'
	if (-f '/usr/libexec/apache/mod_authz_groupfile.so');

    splice @file, $state->{'line'}, 0, @new;
  }

  #
  # All done!  Write the file and exit quietly.
  #
  write_file($file, @file);
  exit 0;

################################################################################
sub read_file($) {
  my $file	= shift || return undef;
  my @rv	= ();

  open FILE, $file or return @rv;
  while (my $l = <FILE>) { chomp $l; push @rv, $l; }
  close FILE;

  return @rv;
}

################################################################################
sub write_file($@) {
  my $file	= shift || return undef;

  open FILE, '>', $file or return undef;
  foreach my $l (@_) { print FILE "$l\n"; }
  close FILE;
}
